/*
 * 定义所有的常量
 */
const DELETE = "delete";

module.exports = exports = {
	RegxType: {
		GLOBAL: "g",
		IGNORE_CASE: "i",
		FORCE: "f",
		MUTLI_LINE: "m"
	},
	XmlEntity: {
		LEFT_ANGLE: "&lt;", // <
		RIGHT_ANGLE: "&gt;",  // >
		AMP: "&amp;", // &
		QUOT: "&quot;", // "
		APOS: "&apos;", // "
		SHARP: "&#35;", // #
		SPACE: "&nbsp;"
	},
	CharCode: {
		ZERO_WIDTH: "\uFEFF",
		ZERO: 0x0030,
		NINE: 0x0039,
		// 大写字母字母的 ASCII 编号
		UPPER_A: 0x0041,
		UPPER_Z: 0x005A,
		UPPER_F: 0x0046,
		// 小写字母字母的 ASCII 编号
		LOWER_A: 0x0061,
		LOWER_Z: 0x007A,
		LOWER_F: 0x0066,
		// 特殊符号的 ASCII 编号
		UNDERBAR: 0x005F, // _
		DOLLAR: 0x0024, // $
	},
	Mark: {
		QUESTION: "?",
		SPACE_REGX: /\s/, // 标准空白
		SPACE: " ", // 标准空白
		TAB: "\t",
		DQUOTE: "\"",
		QUOTE: "'",
		UNDER_SOURCE: "_",
		GRAVE_ACCENT: "`",
		SLASH: "/",
		ASTERISK: "*",
		NEW_LINE: "\n",
		AT: "@",
		AND: "&",
		SHARP: "#",
		COMMA: ",",
		SEMICOLON: ";",
		COLON: ":",
		LEFT_BRACKET: "{",
		RIGHT_BRACKET: "}",
		LEFT_SQUARE_BRACKET: "[",
		RIGHT_SQUARE_BRACKET: "]",
		LEFT_PARENTHE: "(",
		RIGHT_PARENTHE: ")",
		POINT: ".",
		HYPHEN: "-",
		EQUALS: "=",
		TILDE: "~",
		EXCALMATORY: "!",
		LEFT_ANGLE: "<",
		RIGHT_ANGLE: ">"
	},
	MimeType: {
		// application
		JSON: "application/json",
		PDF: "application/pdf",
		JAVASCRIPT: "application/javascript",
		OCTET_STREAM: "application/octet-stream",
		DTD: "application/xml-dtd",
		ZIP: "application/zip",
		// text
		TEXT: "text/plain",
		HTML: "text/html",
		XML: "text/xml",
		CSS: "text/css",
		CSV: "text/csv",
		// image
		GIF: "image/gif",
		PNG: "image/png",
		JPG: "image/jpeg",
		BMP: "image/bmp",
		WEBP: "image/webp",
		ICON: "image/x-icon",
		SVG: "image/svg+xml"
	},
	HttpRequestMethod: {
		GET: "GET",
		PUT: "PUT",
		POST: "POST",
		DELETE: DELETE.toUpperCase(),
		CONNECT: "CONNECT",
		HEAD: "HEAD",
		OPTIONS: "OPTIONS",
		TRACE: "TRACE",
		PATCH: "PATCH"
	},
	HttpStatusCode: {
		CONTINUE: 100,
		SWITCHING_PROTOCOLS: 101,
		PROCESSING: 102,
		OK: 200,
		CREATED: 201,
		ACCEPTED: 202,
		NON_AUTHORITATIVE_INFORMATION: 203,
		NO_CONTENT: 204,
		RESET_CONTENT: 205,
		PARTIAL_CONTENT: 206,
		MULTI_STATUS: 207,
		ALREADY_REPORTED: 208,
		IM_USED: 226,
		MULTIPLE_CHOICES: 300,
		MOVED_PERMANENTLY: 301,
		FOUND: 302,
		SEE_OTHER: 303,
		NOT_MODIFIED: 304,
		USE_PROXY: 305,
		TEMPORARY_REDIRECT: 307,
		PERMANENT_REDIRECT: 308,
		BAD_REQUEST: 400,
		UNAUTHORIZED: 401,
		PAYMENT_REQUIRED: 402,
		FORBIDDEN: 403,
		NOT_FOUND: 404,
		METHOD_NOT_ALLOWED: 405,
		NOT_ACCEPTABLE: 406,
		PROXY_AUTHENTICATION_REQUIRED: 407,
		REQUEST_TIMEOUT: 408,
		CONFLICT: 409,
		GONE: 410,
		LENGTH_REQUIRED: 411,
		PRECONDITION_FAILED: 412,
		PAYLOAD_TOO_LARGE: 413,
		URI_TOO_LONG: 414,
		UNSUPPORTED_MEDIA_TYPE: 415,
		RANGE_NOT_SATISFIABLE: 416,
		EXPECTATION_FAILED: 417,
		I_AM_A_TEAPOT: 418,
		MISDIRECTED_REQUEST: 421,
		UNPROCESSABLE_ENTITY: 422,
		LOCKED: 423,
		FAILED_DEPENDENCY: 424,
		UNORDERED_COLLECTION: 425,
		UPGRADE_REQUIRED: 426,
		PRECONDITION_REQUIRED: 428,
		TOO_MANY_REQUESTS: 429,
		REQUEST_HEADER_FIELDS_TOO_LARGE: 431,
		UNAVAILABLE_FOR_LEGAL_REASONS: 451,
		INTERNAL_SERVER_ERROR: 500,
		NOT_IMPLEMENTED: 501,
		BAD_GATEWAY: 502,
		SERVICE_UNAVAILABLE: 503,
		GATEWAY_TIMEOUT: 504,
		HTTP_VERSION_NOT_SUPPORTED: 505,
		VARIANT_ALSO_NEGOTIATES: 506,
		INSUFFICIENT_STORAGE: 507,
		LOOP_DETECTED: 508,
		BANDWIDTH_LIMIT_EXCEEDED: 509,
		NOT_EXTENDED: 510,
		NETWORK_AUTHENTICATION_REQUIRED: 511
	},
	NumberNonation: {
		HEX: 16,
		DEC: 10,
		OCT: 8,
		BIN: 2
	},
	HtmlTagAttribute: {
		input: {
			BUTTON: "button",
			CHECKBOX: "checkbox",
			COLOR: "color",
			DATE: "date",
			DATETIME_LOCAL: "datetime-local",
			EMAIL: "email",
			FILE: "file",
			HIDDEN: "hidden",
			IMAGE: "image",
			MONTH: "month",
			NUMBER: Number.TYPE_NAME,
			PASSWORD: "password",
			RADIO: "radio",
			RANGE: "range",
			RESET: "reset",
			SEARCH: "search",
			SUBMIT: "submit",
			TEL: "tel",
			TEXT: "text",
			TIME: "time",
			URL: "url",
			WEEK: "week",
		}
	},
	HtmlTag: {
		A: "a",
		ABBR: "abbr",
		ACRONYM: "acronym",
		ADDRESS: "address",
		APPLET: "applet",
		AREA: "area",
		ARTICLE: "article",
		ASIDE: "aside",
		AUDIO: "audio",
		B: "b",
		BASE: "base",
		BASEFONT: "basefont",
		BDI: "bdi",
		BDO: "bdo",
		BIG: "big",
		BLOCKQUOTE: "blockquote",
		BODY: "body",
		BR: "br",
		BUTTON: "button",
		CANVAS: "canvas",
		CAPTION: "caption",
		CENTER: "center",
		CITE: "cite",
		CODE: "code",
		COL: "col",
		COLGROUP: "colgroup",
		COMMAND: "command",
		DATALIST: "datalist",
		DD: "dd",
		DEL: "del",
		DETAILS: "details",
		DFN: "dfn",
		DIALOG: "dialog",
		DIR: "dir",
		DIV: "div",
		DL: "dl",
		DT: "dt",
		EM: "em",
		EMBED: "embed",
		FIELDSET: "fieldset",
		FIGCAPTION: "figcaption",
		FIGURE: "figure",
		FONT: "font",
		FOOTER: "footer",
		FORM: "form",
		FRAME: "frame",
		FRAMESET: "frameset",
		H1: "h1",
		H2: "h2",
		H3: "h3",
		H4: "h4",
		H5: "h5",
		H6: "h6",
		HEAD: "head",
		HEADER: "header",
		HR: "hr",
		HTML: "html",
		I: "i",
		IFRAME: "iframe",
		IMG: "img",
		INPUT: "input",
		INS: "ins",
		KBD: "kbd",
		KEYGEN: "keygen",
		LABEL: "label",
		LEGEND: "legend",
		LI: "li",
		LINK: "link",
		MAP: "map",
		MARK: "mark",
		MENU: "menu",
		META: "meta",
		METER: "meter",
		NAV: "nav",
		NOFRAMES: "noframes",
		NOSCRIPT: "noscript",
		OBJECT: Object.TYPE_NAME,
		OL: "ol",
		OPTGROUP: "optgroup",
		OPTION: "option",
		OUTPUT: "output",
		P: "p",
		PARAM: "param",
		PRE: "pre",
		PROGRESS: "progress",
		Q: "q",
		RP: "rp",
		RT: "rt",
		RUBY: "ruby",
		S: "s",
		SAMP: "samp",
		SCRIPT: "script",
		SECTION: "section",
		SELECT: "select",
		SMALL: "small",
		SOURCE: "source",
		SPAN: "span",
		STRIKE: "strike",
		STRONG: "strong",
		STYLE: "style",
		SUB: "sub",
		SUMMARY: "summary",
		SUP: "sup",
		TABLE: "table",
		TBODY: "tbody",
		TD: "td",
		TEXTAREA: "textarea",
		TFOOT: "tfoot",
		TH: "th",
		THEAD: "thead",
		TIME: "time",
		TITLE: "title",
		TR: "tr",
		TRACK: "track",
		TT: "tt",
		U: "u",
		UL: "ul",
		VAR: "var",
		VIDEO: "video",
		WBR: "wbr"
	}
};